@@ -1,19 +1,3 @@
-# This file is part of CairoSVG
-# Copyright © 2010-2018 Kozea
-#
-# This library is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the Free
-# Software Foundation, either version 3 of the License, or (at your option) any
-# later version.
-#
-# This library is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
-# details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with CairoSVG.  If not, see <http://www.gnu.org/licenses/>.
-
 """
 Surface helpers.
 
@@ -125,6 +109,10 @@ def preserve_ratio(surface, node, width=None, height=None):
         width = width or node_width
         height = height or node_height
         viewbox_width, viewbox_height = node.image_width, node.image_height
+    else:
+        raise TypeError(
+            ('Root node is {}. Should be one of '
+             'marker, svg, image, or g.').format(node.tag))
 
     translate_x = 0
     translate_y = 0
@@ -205,17 +193,57 @@ def rotate(x, y, angle):
     return x * cos(angle) - y * sin(angle), y * cos(angle) + x * sin(angle)
 
 
-def transform(surface, string, gradient=None):
+def transform(surface, transform_string, gradient=None, transform_origin=None):
     """Transform ``surface`` or ``gradient`` if supplied using ``string``.
 
     See http://www.w3.org/TR/SVG/coords.html#TransformAttribute
 
     """
-    if not string:
+    if not transform_string:
         return
 
-    transformations = re.findall(r'(\w+) ?\( ?(.*?) ?\)', normalize(string))
+    transformations = re.findall(
+        r'(\w+) ?\( ?(.*?) ?\)', normalize(transform_string))
     matrix = cairo.Matrix()
+
+    if transform_origin:
+        origin = transform_origin.split(' ')
+        origin_x = origin[0]
+        if len(origin) == 1:
+            if origin_x in ('top', 'bottom'):
+                origin_y = origin_x
+                origin_x = surface.width / 2
+            else:
+                origin_y = surface.height / 2
+        elif len(origin) > 1:
+            if origin_x in ('top', 'bottom'):
+                origin_y = origin_x
+                origin_x = origin[1]
+            else:
+                origin_y = origin[1]
+        else:
+            return
+
+        if origin_x == 'center':
+            origin_x = surface.width / 2
+        elif origin_x == 'left':
+            origin_x = 0
+        elif origin_x == 'right':
+            origin_x = surface.width
+        else:
+            origin_x = size(surface, origin_x, 'x')
+
+        if origin_y == 'center':
+            origin_y = surface.height / 2
+        elif origin_y == 'top':
+            origin_y = 0
+        elif origin_y == 'bottom':
+            origin_y = surface.height
+        else:
+            origin_y = size(surface, origin_y, 'y')
+
+        matrix.translate(float(origin_x), float(origin_y))
+
     for transformation_type, transformation in transformations:
         values = [size(surface, value) for value in transformation.split(' ')]
         if transformation_type == 'matrix':
@@ -240,16 +268,10 @@ def transform(surface, string, gradient=None):
             if len(values) == 1:
                 values = 2 * values
             matrix.scale(*values)
-    apply_matrix_transform(surface, matrix, gradient)
 
+    if transform_origin:
+        matrix.translate(-float(origin_x), -float(origin_y))
 
-def apply_matrix_transform(surface, matrix, gradient=None):
-    """Apply a ``matrix`` to ``surface`` or ``gradient`` if supplied.
-
-    When the matrix is not invertible, this function clips the context to an
-    empty path instead of raising an exception.
-
-    """
     try:
         matrix.invert()
     except cairo.Error:
@@ -356,6 +378,9 @@ def size(surface, string, reference='xy'):
     elif string.endswith('ex'):
         # Assume that 1em == 2ex
         return surface.font_size * float(string[:-2]) / 2
+    elif string.endswith('ch'):
+        # A '0' must be assumed to be 0.5em wide.
+        return surface.font_size * float(string[:-2]) / 2
 
     for unit, coefficient in UNITS.items():
         if string.endswith(unit):
